@@ -2,7 +2,7 @@
 /* $Id$ */
 
 /*  libticalcs - Ti Calculator library, a part of the TiLP project
- *  Copyright (C) 1999-2005  Romain Livin
+ *  Copyright (C) 1999-2005  Romain Liévin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,9 +31,9 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <ticonv.h>
-#include "gettext.h"
 #include "ticalcs.h"
+#include "internal.h"
+#include "gettext.h"
 #include "logging.h"
 
 /* Dirlist format:
@@ -60,7 +60,8 @@
 
 static gboolean free_varentry(GNode* node, gpointer data)
 {
-	if(node && node->data)
+	(void)data;
+	if (node && node->data)
 	{
 		VarEntry* ve = node->data;
 		tifiles_ve_delete(ve);
@@ -83,19 +84,86 @@ TIEXPORT3 void TICALL ticalcs_dirlist_destroy(GNode** tree)
 	{
 		TreeInfo *ti;
 
-		if((*tree)->children != NULL)
+		if ((*tree)->children != NULL)
 		{
 			g_node_traverse(*tree, G_IN_ORDER, G_TRAVERSE_LEAVES, -1, free_varentry, NULL);
 		}
 
 		ti = (TreeInfo *)((*tree)->data);
-		g_free((*tree)->data);
+		g_free(ti);
 		g_node_destroy(*tree);
 
 		*tree = NULL;
 	}
 }
 
+static void display_node(TreeInfo * info, VarEntry * ve, unsigned char type)
+{
+	char * utf8 = ticonv_varname_to_utf8(info->model, ve->name, type);
+	int k;
+
+	printf("| ");
+	for (k = 0; k < 8; k++)
+	{
+		printf("%02X", (uint8_t) (ve->name)[k]);
+	}
+	printf(" | ");
+	printf("%8s", utf8);
+	printf(" | ");
+	printf("%2i", ve->attr);
+	printf(" | ");
+	printf("%02X", ve->type);
+	printf(" | ");
+	printf("%08X", ve->size);
+	printf(" | ");
+	printf("%8s", ve->folder);
+	printf(" |");
+	printf("\n");
+
+	ticonv_utf8_free(utf8);
+}
+
+static void display_tree(TreeInfo * info, GNode * vars)
+{
+	int i;
+
+	//ticalcs_info("vars has %d children", (int)g_node_n_children(vars));
+	for (i = 0; i < (int)g_node_n_children(vars); i++) // parse children
+	{
+		GNode *parent = g_node_nth_child(vars, i);
+		VarEntry *fe = (VarEntry *) (parent->data);
+		int j;
+
+		if (fe != NULL)
+		{
+			display_node(info, fe, -1);
+		}
+
+		//ticalcs_info("parent has %d children", (int)g_node_n_children(parent));
+		if (info->model != CALC_NSPIRE)
+		{
+			for (j = 0; j < (int)g_node_n_children(parent); j++)
+			{
+				GNode *child = g_node_nth_child(parent, j);
+				VarEntry *ve = (VarEntry *) (child->data);
+
+				display_node(info, ve, ve->type);
+			}
+		}
+		else
+		{
+			// Recurse into sub-folder if necessary.
+			if (fe != NULL && fe->type == NSP_DIR)
+			{
+				//ticalcs_info("Recurse");
+				display_tree(info, parent);
+			}
+		}
+		//ticalcs_info("finished enumerating %d children of parent", (int)g_node_n_children(parent));
+	}
+	//ticalcs_info("finished enumerating %d children of vars", (int)g_node_n_children(vars));
+}
+
 /**
  * ticalcs_dirlist_display:
  * @tree: the tree to display.
@@ -108,8 +176,7 @@ TIEXPORT3 void TICALL ticalcs_dirlist_display(GNode* tree)
 {
 	GNode *vars = tree;
 	TreeInfo *info;
-	int i, j, k;
-	char *utf8;
+	int i;
 
 	if (tree == NULL)
 	{
@@ -119,75 +186,31 @@ TIEXPORT3 void TICALL ticalcs_dirlist_display(GNode* tree)
 
 	info = (TreeInfo *)(tree->data);
 	if (info == NULL)
-		return;
-
-  printf(  "+------------------+----------+----+----+----------+----------+\n");
-  printf(_("| B. name          | T. name  |Attr|Type| Size     | Folder   |\n"));
-  printf(  "+------------------+----------+----+----+----------+----------+\n");
-
-  for (i = 0; i < (int)g_node_n_children(vars); i++)	// parse folders
-  {
-    GNode *parent = g_node_nth_child(vars, i);
-    VarEntry *fe = (VarEntry *) (parent->data);
-
-    if (fe != NULL) 
 	{
-		utf8 = ticonv_varname_to_utf8(info->model, fe->name, -1);
+		return;
+	}
 
-      printf("| ");
-      for (k = 0; k < 8; k++)
-		printf("%02X", (uint8_t) (fe->name)[k]);
-      printf(" | ");	
-      printf("%8s", utf8);
-      printf(" | ");
-      printf("%2i", fe->attr);
-      printf(" | ");
-      printf("%02X", fe->type);
-      printf(" | ");
-      printf("%08X", fe->size);
-      printf(" | ");
-      printf("%8s", fe->folder);
-      printf(" |");
-	  printf("\n");
+	printf(  "+------------------+----------+----+----+----------+----------+\n");
+	printf(_("| B. name          | T. name  |Attr|Type| Size     | Folder   |\n"));
+	printf(  "+------------------+----------+----+----+----------+----------+\n");
 
-	  g_free(utf8);
-    }
+	i = (int)g_node_n_children(vars);
+	//ticalcs_info("Root has %d children", i);
+	display_tree(info, vars);
+	if (!i)
+	{
+		if (!strcmp(info->type, VAR_NODE_NAME))
+		{
+			printf(_("| No variables     |\n"));
+		}
+		else if (!strcmp(info->type, APP_NODE_NAME))
+		{
+			printf(_("| No applications  |\n"));
+		}
+	}
 
-    for (j = 0; j < (int)g_node_n_children(parent); j++)	//parse variables
-    {
-      GNode *child = g_node_nth_child(parent, j);
-      VarEntry *ve = (VarEntry *) (child->data);
-
-	  utf8 = ticonv_varname_to_utf8(info->model, ve->name, ve->type);
-
-      printf("| ");
-      for (k = 0; k < 8; k++) 
-		printf("%02X", (uint8_t) (ve->name)[k]);
-      printf(" | ");
-      printf("%8s", utf8);
-      printf(" | ");
-      printf("%2i", ve->attr);
-      printf(" | ");
-      printf("%02X", ve->type);
-      printf(" | ");
-      printf("%08X", ve->size);
-      printf(" | ");
-      printf("%8s", ve->folder);
-      printf(" |");
-	  printf("\n");
-
-	  g_free(utf8);
-    }
-  }
-  if (!i)
-  {
-	  if(!strcmp(info->type, VAR_NODE_NAME))
-		printf(_("| No variables     |\n"));
-	  else if(!strcmp(info->type, APP_NODE_NAME))
-		printf(_("| No applications  |\n"));
-  }
-  printf(_("+------------------+----------+----+----+----------+----------+"));
-  printf("\n");
+	printf(_("+------------------+----------+----+----+----------+----------+"));
+	printf("\n");
 }
 
 /**
@@ -213,10 +236,14 @@ TIEXPORT3 VarEntry *TICALL ticalcs_dirlist_ve_exist(GNode* tree, VarEntry *s)
 
 	info = (TreeInfo *)(tree->data);
 	if (info == NULL)
+	{
 		return NULL;
+	}
 
 	if (strcmp(info->type, VAR_NODE_NAME) && strcmp(info->type, APP_NODE_NAME))
+	{
 		return NULL;
+	}
 
 	for (i = 0; i < (int)g_node_n_children(vars); i++)	// parse folders
 	{
@@ -224,7 +251,9 @@ TIEXPORT3 VarEntry *TICALL ticalcs_dirlist_ve_exist(GNode* tree, VarEntry *s)
 		VarEntry *fe = (VarEntry *) (parent->data);
 
 		if ((fe != NULL) && strcmp(fe->name, s->folder))
+		{
 			continue;
+		}
 
 		for (j = 0; j < (int)g_node_n_children(parent); j++)	//parse variables
 		{
@@ -232,8 +261,10 @@ TIEXPORT3 VarEntry *TICALL ticalcs_dirlist_ve_exist(GNode* tree, VarEntry *s)
 			VarEntry *ve = (VarEntry *) (child->data);
 
 			if (   !strcmp(ve->name, s->name)
-			    && (   !(info->model >= CALC_TI73 && info->model <= CALC_TI84P)
-			        || (ve->type == s->type)))
+			    && (   !((info->model >= CALC_TI73 && info->model <= CALC_TI84P)
+			        || info->model == CALC_TI84P_USB || info->model == CALC_TI83PCE_USB
+			        || info->model == CALC_TI84PCE_USB || info->model == CALC_TI82A_USB)
+			        || info->model == CALC_TI84PT_USB || (ve->type == s->type)))
 			{
 				return ve;
 			}
@@ -251,11 +282,11 @@ TIEXPORT3 VarEntry *TICALL ticalcs_dirlist_ve_exist(GNode* tree, VarEntry *s)
  *
  * Return value: the number of entries.
  **/
-TIEXPORT3 int TICALL ticalcs_dirlist_ve_count(GNode* tree)
+TIEXPORT3 unsigned int TICALL ticalcs_dirlist_ve_count(GNode* tree)
 {
-	int i, j;
+	unsigned int i, j;
 	GNode *vars = tree;
-	int nvars = 0;
+	unsigned int nvars = 0;
 	TreeInfo *info;
 
 	if (tree == NULL)
@@ -266,17 +297,23 @@ TIEXPORT3 int TICALL ticalcs_dirlist_ve_count(GNode* tree)
 
 	info = (TreeInfo *)(tree->data);
 	if (info == NULL)
+	{
 		return 0;
+	}
 
 	if (strcmp(info->type, VAR_NODE_NAME) && strcmp(info->type, APP_NODE_NAME))
+	{
 		return 0;
+	}
 
-	for (i = 0; i < (int)g_node_n_children(vars); i++)	// parse folders
+	for (i = 0; i < g_node_n_children(vars); i++)	// parse folders
 	{
 		GNode *parent = g_node_nth_child(vars, i);
 
-		for (j = 0; j < (int)g_node_n_children(parent); j++)	//parse variables
+		for (j = 0; j < g_node_n_children(parent); j++)	//parse variables
+		{
 			nvars++;
+		}
 	}
 
 	return nvars;
@@ -305,10 +342,14 @@ TIEXPORT3 int TICALL ticalcs_dirlist_ram_used(GNode* tree)
 
 	info = (TreeInfo *)(tree->data);
 	if (info == NULL)
+	{
 		return 0;
+	}
 
 	if (strcmp(info->type, VAR_NODE_NAME))
+	{
 		return 0;
+	}
 	
 	for (i = 0; i < (int)g_node_n_children(vars); i++)	// parse folders
 	{
@@ -319,8 +360,10 @@ TIEXPORT3 int TICALL ticalcs_dirlist_ram_used(GNode* tree)
 			GNode *child = g_node_nth_child(parent, j);
 			VarEntry *ve = (VarEntry *) (child->data);
 
-			if(ve->attr != ATTRB_ARCHIVED)
+			if (ve->attr != ATTRB_ARCHIVED)
+			{
 				mem += ve->size;
+			}
 		}
 	}
 
@@ -342,7 +385,7 @@ TIEXPORT3 int TICALL ticalcs_dirlist_flash_used(GNode* vars, GNode* apps)
 	TreeInfo *info1;
 	TreeInfo *info2;
 
-	if (!vars || !apps)
+	if (vars == NULL || apps == NULL)
 	{
 		ticalcs_critical("ticalcs_dirlist_flash_used: an argument is NULL");
 		return 0;
@@ -351,7 +394,9 @@ TIEXPORT3 int TICALL ticalcs_dirlist_flash_used(GNode* vars, GNode* apps)
 	info1 = (TreeInfo *)(vars->data);
 	info2 = (TreeInfo *)(apps->data);
 	if (info1 == NULL ||  info2 == NULL)
+	{
 		return 0;
+	}
 
 	if (!strcmp(info1->type, VAR_NODE_NAME))
 	{
@@ -364,8 +409,10 @@ TIEXPORT3 int TICALL ticalcs_dirlist_flash_used(GNode* vars, GNode* apps)
 				GNode *child = g_node_nth_child(parent, j);
 				VarEntry *ve = (VarEntry *) (child->data);
 
-				if(ve->attr == ATTRB_ARCHIVED)
+				if (ve->attr == ATTRB_ARCHIVED)
+				{
 					mem += ve->size;
+				}
 			}
 		}
 	}
@@ -420,25 +467,33 @@ TIEXPORT3 void TICALL ticalcs_dirlist_ve_add(GNode* tree, VarEntry *entry)
 
 	info = (TreeInfo *)(tree->data);
 	if (info == NULL)
+	{
 		return;
+	}
 
 	if (strcmp(info->type, VAR_NODE_NAME) && strcmp(info->type, APP_NODE_NAME))
+	{
 		return;
+	}
 
-	if(!strcmp(entry->folder, "") && tifiles_has_folder(info->model))
+	if (!strcmp(entry->folder, "") && tifiles_has_folder(info->model))
+	{
 		folder = "main";
+	}
 	else
+	{
 		folder = entry->folder;
+	}
 
 	// If TI8x tree is empty, create pseudo-folder (NULL)
-	if(!g_node_n_children(tree) && !tifiles_has_folder(info->model))
+	if (!g_node_n_children(tree) && !tifiles_has_folder(info->model))
 	{
 		parent = g_node_new(NULL);
 		g_node_append(tree, parent);
 	}
 
 	// Check for one folder at least...
-	if(g_node_n_children(tree) > 0)
+	if (g_node_n_children(tree) > 0)
 	{
 		// Parse folders
 		for (found = 0, i = 0; i < (int)g_node_n_children(tree); i++)
@@ -446,25 +501,27 @@ TIEXPORT3 void TICALL ticalcs_dirlist_ve_add(GNode* tree, VarEntry *entry)
 			parent = g_node_nth_child(tree, i);
 			fe = (VarEntry *) (parent->data);
 
-			if(fe == NULL)
+			if (fe == NULL)
+			{
 				break;
+			}
 
-			if(!strcmp(fe->name, folder))
+			if (!strcmp(fe->name, folder))
 			{
 				found = !0;
 				break;
 			}
 		}
 	}
-		
+
 	// folder doesn't exist? => create!
-	if((!found && fe) || 
-	   (!g_node_n_children(tree) && tifiles_has_folder(info->model)))
+	if ((!found && fe) ||
+	    (!g_node_n_children(tree) && tifiles_has_folder(info->model)))
 	{
 		fe = tifiles_ve_create();
 		if (fe != NULL)
 		{
-			strcpy(fe->name, entry->folder);
+			ticalcs_strlcpy(fe->name, entry->folder, sizeof(fe->name));
 			fe->type = TI89_DIR;
 
 			parent = g_node_new(fe);
@@ -472,23 +529,25 @@ TIEXPORT3 void TICALL ticalcs_dirlist_ve_add(GNode* tree, VarEntry *entry)
 		}
 	}
 
-	if(!strcmp(entry->name, ""))
+	if (!strcmp(entry->name, ""))
+	{
 		return;
+	}
 
 	// next, add variables beneath this folder
-	for(found = 0, j = 0; j < (int)g_node_n_children(parent); j++)
+	for (found = 0, j = 0; j < (int)g_node_n_children(parent); j++)
 	{
 		child = g_node_nth_child(parent, j);
 		ve = (VarEntry *) (child->data);
 
-		if(!strcmp(ve->name, entry->name))
+		if (!strcmp(ve->name, entry->name))
 		{
 			found = !0;
 			break;
 		}
 	}
 
-	if(!found)
+	if (!found)
 	{
 		ve = tifiles_ve_dup(entry);
 		if (ve != NULL)
@@ -498,8 +557,10 @@ TIEXPORT3 void TICALL ticalcs_dirlist_ve_add(GNode* tree, VarEntry *entry)
 		}
 	}
 
-	if(fe && found)
+	if (fe && found)
+	{
 		fe->size++;
+	}
 }
 
 
@@ -534,15 +595,23 @@ TIEXPORT3 void TICALL ticalcs_dirlist_ve_del(GNode* tree, VarEntry *entry)
 
 	info = (TreeInfo *)(tree->data);
 	if (info == NULL)
+	{
 		return;
+	}
 
 	if (strcmp(info->type, VAR_NODE_NAME))
+	{
 		return;
+	}
 
-	if(!strcmp(entry->folder, "") && tifiles_has_folder(info->model))
+	if (!strcmp(entry->folder, "") && tifiles_has_folder(info->model))
+	{
 		folder = "main";
+	}
 	else
+	{
 		folder = entry->folder;
+	}
 
 	// Parse folders
 	for (found = 0, i = 0; i < (int)g_node_n_children(tree); i++)
@@ -550,38 +619,44 @@ TIEXPORT3 void TICALL ticalcs_dirlist_ve_del(GNode* tree, VarEntry *entry)
 		parent = g_node_nth_child(tree, i);
 		fe = (VarEntry *) (parent->data);
 
-		if(fe == NULL)
+		if (fe == NULL)
+		{
 			break;
+		}
 
-		if(!strcmp(fe->name, folder))
+		if (!strcmp(fe->name, folder))
 		{
 			found = !0;
 			break;
 		}
 	}
 
-	if(!found && fe)
+	if (!found && fe)
+	{
 		return;
+	}
 
 	// next, delete variables beneath this folder
-	for(found = 0, j = 0; j < (int)g_node_n_children(parent); j++)
+	for (found = 0, j = 0; j < (int)g_node_n_children(parent); j++)
 	{
 		child = g_node_nth_child(parent, j);
 		ve = (VarEntry *) (child->data);
 
-		if(!strcmp(ve->name, entry->name))
+		if (!strcmp(ve->name, entry->name))
 		{
 			found = !0;
 			break;
 		}
 	}
 
-	if(found)
+	if (found)
 	{
 		tifiles_ve_delete(child->data);
 		g_node_destroy(child);
 	}
 
-	if(fe && found)
+	if (fe && found)
+	{
 		fe->size--;
+	}
 }
